@@ -3,11 +3,13 @@ This module contains the main disease modelling class.
 """
 import copy
 import logging
+from datetime import datetime
 from collections import OrderedDict
 from typing import Callable, Dict, List, Optional, Tuple
 
 import networkx
 import numpy as np
+import pandas as pd
 
 import summer.flows as flows
 from summer import stochastic
@@ -18,7 +20,7 @@ from summer.derived_outputs import DerivedOutputRequest, calculate_derived_outpu
 from summer.runner import ReferenceRunner, VectorizedRunner
 from summer.solver import SolverType, solve_ode
 from summer.stratification import Stratification
-from summer.utils import get_scenario_start_index
+from summer.utils import get_scenario_start_index, ref_times_to_dti
 
 logger = logging.getLogger()
 
@@ -66,6 +68,7 @@ class CompartmentalModel:
         compartments: List[str],
         infectious_compartments: List[str],
         timestep: float = 1.0,
+        ref_date: datetime = None
     ):
         start_t, end_t = times
         assert end_t > start_t, "End time must be greater than start time"
@@ -78,6 +81,9 @@ class CompartmentalModel:
         self.times = np.linspace(start_t, end_t, num=int(num_steps))
         self.timestep = timestep
 
+        # Set the ref_date; the datetime object equivalent to times[0]
+        self.ref_date = ref_date
+
         error_msg = "Infectious compartments must be a subset of compartments"
         assert all(n in compartments for n in infectious_compartments), error_msg
         self.compartments = [Compartment(n) for n in compartments]
@@ -566,7 +572,7 @@ class CompartmentalModel:
         expected_count: Optional[int], new_flows: List[flows.BaseFlow]
     ):
         """
-        Ensure the number of new flows created is the expected amountmodel
+        Ensure the number of new flows created is the expected amount
 
         """
         if expected_count is not None:
@@ -592,7 +598,8 @@ class CompartmentalModel:
         if not len(strata):
             return name_query
         else:
-            return [c for c in name_query if c.has_strata(strata)]
+            _strata = frozenset(strata.items())
+            return [c for c in name_query if c._has_strata(_strata)]
 
     """
     Stratifying the model
@@ -606,12 +613,22 @@ class CompartmentalModel:
             strat: The stratification to apply.
 
         """
+        # Enable/disable runtime assertions for strat
+        strat._validate = self._should_validate
+
         # Validate flow adjustments
         flow_names = [f.name for f in self._flows]
         for n in strat.flow_adjustments.keys():
             msg = f"Flow adjustment for '{n}' refers to a flow that is not present in the model."
             assert n in flow_names, msg
 
+        for fadj in strat.flow_adjustments.values():
+            for _, source_strata, dest_strata in fadj:
+                # Verify source
+                self._strata_exist(source_strata)
+                # Veryify dest
+                self._strata_exist(dest_strata)
+
         # Validate infectiousness adjustments.
         msg = "All stratification infectiousness adjustments must refer to a compartment that is present in model."
         assert all(
@@ -690,6 +707,20 @@ class CompartmentalModel:
 
         self._stratifications.append(strat)
 
+    def _strata_exist(self, strata: dict):
+        """
+        Verify whether all the strata exist within the model
+        Raises an Exception if not
+        """
+        strat_names = [s.name for s in self._stratifications]
+        for k, v in strata.items():
+            if k not in strat_names:
+                raise KeyError(f"Invalid stratification {k}")
+            for s in self._stratifications:
+                if k == s.name:
+                    if v not in s.strata:
+                        raise ValueError(f"Invalid stratum {v} for {s}")
+
     """
     Running the model
     """
@@ -729,6 +760,9 @@ class CompartmentalModel:
             # Run the model as a deterministic ODE
             self._solve_ode(solver, kwargs)
 
+        # Constrain outputs; interpolation from solvers can produce negative (invalid) values
+        self.outputs[self.outputs < 0.0] = 0.0
+
         # Calculate any requested derived outputs, based on the calculated compartment sizes.
         self.derived_outputs, self._derived_outputs_idx_cache = self._calculate_derived_outputs()
 
@@ -1145,3 +1179,19 @@ class CompartmentalModel:
             system (AdjustmentSystem): AdjustmentSystem object containing the implementation details
         """
         self._adjustment_systems[name] = system
+
+    def _get_ref_idx(self):
+        if self.ref_date:
+            times = ref_times_to_dti(self.ref_date, self.times)
+        else:
+            times = self.times
+        return times
+
+    def get_outputs_df(self):
+        idx = self._get_ref_idx()
+        column_str = [str(c) for c in self.compartments]
+        return pd.DataFrame(self.outputs, index=idx, columns=column_str)
+
+    def get_derived_outputs_df(self):
+        idx = self._get_ref_idx()
+        return pd.DataFrame(self.derived_outputs, index=idx)
