@@ -47,9 +47,16 @@ class Stratification:
         self.flow_adjustments = {}
         self.infectiousness_adjustments = {}
 
+        # Store internal fast lookup table for flow adjustments using frozenset
+        self._flow_adjustments_fs = {}
+
         # No heterogeneous mixing matrix by default.
         self.mixing_matrix = None
 
+        # Enable/disable validation (runtime assertions)
+        # Not directly user accessable - is toggled by CompartmentalModel
+        self._validate = True
+
     def __repr__(self):
         return f"Stratification: {self.name}"
 
@@ -83,7 +90,7 @@ class Stratification:
         assert abs(1 - sum(proportions.values())) < COMP_SPLIT_REQUEST_ERROR, msg
         self.population_split = proportions
 
-    def add_flow_adjustments(
+    def set_flow_adjustments(
         self,
         flow_name: str,
         adjustments: Dict[str, Adjustment],
@@ -91,11 +98,12 @@ class Stratification:
         dest_strata: Optional[Dict[str, str]] = None,
     ):
         """
-        Add an adjustment of a flow to the stratification.
+        Set an adjustment of a flow to the stratification.
+        Adjustments from previous stratifications will be applied before this.
         You can use time-varying functions for infectiousness adjustments.
 
-        It is possible to specify multiple conflicting flow adjustments for the same flow.
-        In this case, only the last-created applicable adjustment will be chosen.
+        It is possible to specify multiple flow adjustments for the same flow in a given Stratification.
+        In this case, only the last-created applicable adjustment will be applied.
 
         Args:
             flow_name: The name of the flow to adjust.
@@ -111,7 +119,7 @@ class Stratification:
                     strata=["urban", "rural", "alpine"],
                     compartments=["S", "I", "R"]
                 )
-                strat.add_flow_adjustments("recovery", {
+                strat.set_flow_adjustments("recovery", {
                     "urban": Multiply(1.5),
                     "rural": Multiply(0.8),
                     "alpine": None, # No adjustment
@@ -131,6 +139,9 @@ class Stratification:
             ]
         ), msg
 
+        msg = "Cannot add new flow adjustments after stratification has already been applied"
+        assert len(self._flow_adjustments_fs) == 0, msg
+
         if flow_name not in self.flow_adjustments:
             self.flow_adjustments[flow_name] = []
 
@@ -148,24 +159,36 @@ class Stratification:
         flow_adjustments = self.flow_adjustments.get(flow.name, [])
         matching_adjustment = None
 
+        # Cache frozenset lookup
+        if flow.name not in self._flow_adjustments_fs:
+            self._flow_adjustments_fs[flow.name] = cur_fadj_fs = []
+            for adjustment, source_strata, dest_strata in flow_adjustments:
+                cur_fadj_fs.append( (adjustment, source_strata, frozenset(source_strata.items()), dest_strata, frozenset(dest_strata.items())) )
+        
+        flow_adj_fs = self._flow_adjustments_fs[flow.name]
+
         # Loop over all the requested adjustments.
-        for adjustment, source_strata, dest_strata in flow_adjustments:
+        for adjustment, source_strata, _source_strata, dest_strata, _dest_strata in flow_adj_fs:
 
-            # For entry flows:
-            msg = f"Source strata requested in flow adjustment of {self.name}, but {flow.name} does not have a source"
-            assert not (source_strata and not flow.source), msg
+            # These are expensive assertions and can be disabled in multi-run situations
+            if self._validate:
+                # For entry flows:
+                msg = f"Source strata requested in flow adjustment of {self.name}, but {flow.name} does not have a source"
+                assert not (source_strata and not flow.source), msg
 
-            # For exit flows:
-            msg = f"Dest strata requested in flow adjustment of {self.name}, but {flow.name} does not have a dest"
-            assert not (dest_strata and not flow.dest), msg
+                # For exit flows:
+                msg = f"Dest strata requested in flow adjustment of {self.name}, but {flow.name} does not have a dest"
+                assert not (dest_strata and not flow.dest), msg
 
             # Make sure that the source request applies to this flow because it has all of the requested strata.
             # Note that these can be specified in the current or any previous stratifications.
             is_source_no_match = (
-                source_strata and flow.source and not flow.source.has_strata(source_strata)
+                source_strata and flow.source and not flow.source._has_strata(_source_strata)
             )
-            is_dest_no_match = dest_strata and flow.dest and not flow.dest.has_strata(dest_strata)
-            if is_source_no_match or is_dest_no_match:
+            if is_source_no_match:
+                continue
+            is_dest_no_match = dest_strata and flow.dest and not flow.dest._has_strata(_dest_strata)
+            if is_dest_no_match:
                 continue
 
             matching_adjustment = adjustment
